@@ -19,16 +19,16 @@ from urllib.parse import urljoin
 from urllib.request import pathname2url
 
 import numpy as np
-import sip
 
 from AnyQt.QtCore import Qt, QObject, QFile, QTimer, QUrl, QSize, QEventLoop, \
     pyqtProperty, pyqtSlot, pyqtSignal
 from AnyQt.QtGui import QColor
 from AnyQt.QtWidgets import QSizePolicy, QWidget, qApp
+from AnyQt import sip
 
 from orangewidget.i18n_config import *
 def __(key):
-    return i18n.t("widget.svgexporter." + key)
+    return i18n.t("widget.webview.warn." + key)
 try:
     from AnyQt.QtWebKitWidgets import QWebView
     HAVE_WEBKIT = True
@@ -106,7 +106,7 @@ if HAVE_WEBENGINE:
             if debug:
                 port = os.environ.setdefault('QTWEBENGINE_REMOTE_DEBUGGING', '12088')
                 warnings.warn(
-                    __('warn.set environment').format(port=port))
+                    __("set_environment").format(port=port))
             super().__init__(parent,
                              sizeHint=QSize(500, 400),
                              sizePolicy=QSizePolicy(QSizePolicy.Expanding,
@@ -433,6 +433,10 @@ if HAVE_WEBKIT:
 
     class WebviewWidget(_WebViewBase, WebKitView):
         def __init__(self, parent=None, bridge=None, *, debug=False, **kwargs):
+            # WebEngine base WebviewWidget has js_timeout parameter, since
+            # user do not know which one will get and passing js_timeout to
+            # WebKitView causes error we should remove
+            kwargs.pop("js_timeout", None)
             WebKitView.__init__(self, parent, bridge, debug=debug, **kwargs)
             _WebViewBase.__init__(self)
 
@@ -526,7 +530,7 @@ elif HAVE_WEBENGINE:
             id = next(self._id_gen)
             value = self._objects[id] = dict(id=id, name=name, obj=obj)
             # Wait till JS is connected to receive objects
-            wait(until=lambda: self.receivers(self.objectChanged))
+            wait(until=lambda: self.receivers(self.objectChanged), timeout=30000)
             self.objectChanged.emit(value)
 
         @pyqtSlot(int)
