@@ -9,7 +9,7 @@ from math import log10
 from typing import Optional, Union, List
 
 from AnyQt.QtWidgets import (
-    QWidget, QDialog, QVBoxLayout, QSizePolicy, QApplication, QStyle,
+    QWidget, QDialog, QVBoxLayout, QSizePolicy, QStyle,
     QShortcut, QSplitter, QSplitterHandle, QPushButton, QStatusBar,
     QProgressBar, QAction, QFrame, QStyleOption, QWIDGETSIZE_MAX,
     QHBoxLayout)
@@ -29,7 +29,7 @@ from orangewidget.utils.messages import (
     WidgetMessagesMixin, UnboundMsg, MessagesWidget
 )
 from orangewidget.utils.signals import (
-    WidgetSignalsMixin, Input, Output, AttributeList,
+    WidgetSignalsMixin, Input, Output, MultiInput,
     InputSignal, OutputSignal,
     Default, NonDefault, Single, Multiple, Dynamic, Explicit
 )  # pylint: disable=unused-import
@@ -46,8 +46,8 @@ def __(key):
 
 
 __all__ = [
-    "OWBaseWidget", "Input", "Output", "AttributeList", "Message", "Msg",
-    "StateInfo",
+    "OWBaseWidget", "Input", "Output", "MultiInput",
+    "Message", "Msg", "StateInfo",
 ]
 
 
@@ -250,13 +250,14 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
     #: should it not already be part of a package.
     category: str = None
 
+    #: Ratio between width and height for mainArea widgets,
+    #: set to None for super().sizeHint()
+    mainArea_width_height_ratio: Optional[float] = 1.1
+
     # -------------------------------------------------------------------------
     # Private Interface
     # -------------------------------------------------------------------------
 
-    # Global widget count
-    widget_id = 0
-
     # Custom widget id, kept for backward compatibility
     id = None
 
@@ -307,9 +308,6 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
         self.graphButton = None
         self.report_button = None
 
-        OWBaseWidget.widget_id += 1
-        self.widget_id = OWBaseWidget.widget_id
-
         captionTitle = self.name if captionTitle is None else captionTitle
 
         # must be set without invoking setCaption
@@ -343,6 +341,7 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
         self.__splitter = None
         if self.want_basic_layout:
             self.set_basic_layout()
+            self.update_summaries()
 
         sc = QShortcut(QKeySequence(Qt.ShiftModifier | Qt.Key_F1), self)
         sc.activated.connect(self.__quicktip)
@@ -374,8 +373,9 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
                 warnings.warn(
                     "subclassing of widget classes is deprecated and will be "
                     "disabled in the future.\n"
-                    f"Extract code from {base.__name__} or explicitly open it.",
-                    RuntimeWarning)
+                    f"Extract code from {base.__name__} or explicitly open it "
+                    "by adding `openclass=True` to class definition.",
+                    RuntimeWarning, stacklevel=3)
                 # raise TypeError(f"class {base.__name__} cannot be subclassed")
 
     @classmethod
@@ -457,8 +457,8 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
                     x0, x1 = 6, w
                 else:
                     x0, x1 = w, 6
-                y = self.height() / 2
-                h = (w - 6) / 1.12
+                y = self.height() // 2
+                h = int((w - 6) / 1.12)
                 painter.setRenderHint(painter.Antialiasing)
                 painter.drawLines(
                     QLine(x0, y - h, x1, y),
@@ -480,7 +480,7 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
         self.layout().addWidget(self.__splitter)
 
     def _insert_control_area(self):
-        self.left_side = gui.vBox(self.__splitter, addSpace=0)
+        self.left_side = gui.vBox(self.__splitter, spacing=0)
         if self.want_main_area:
             self.left_side.setSizePolicy(
                 QSizePolicy.MinimumExpanding, QSizePolicy.MinimumExpanding)
@@ -488,28 +488,34 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
             scroll_area = VerticalScrollArea(self.left_side)
             scroll_area.setSizePolicy(QSizePolicy.MinimumExpanding,
                                       QSizePolicy.Preferred)
-            self.controlArea = gui.vBox(scroll_area, spacing=0,
-                                        sizePolicy=(QSizePolicy.Fixed,
-                                                    QSizePolicy.MinimumExpanding))
+            self.controlArea = gui.vBox(scroll_area, spacing=6,
+                                        sizePolicy=(QSizePolicy.MinimumExpanding,
+                                                    QSizePolicy.Preferred))
             scroll_area.setWidget(self.controlArea)
 
             self.left_side.layout().addWidget(scroll_area)
 
             m = 4, 4, 0, 4
         else:
-            self.controlArea = gui.vBox(self.left_side, spacing=0)
+            self.controlArea = gui.vBox(self.left_side, spacing=6)
 
             m = 4, 4, 4, 4
 
         if self.buttons_area_orientation is not None:
             self._insert_buttons_area()
-            self.buttonsArea.layout().setContentsMargins(4, 4, 0, 4)
+            self.buttonsArea.layout().setContentsMargins(
+                m[0] + 8, m[1], m[2] + 8, m[3]
+            )
+            # margins are nice on macOS with this
+            m = m[0], m[1], m[2], m[3] - 2
 
         self.controlArea.layout().setContentsMargins(*m)
 
     def _insert_buttons_area(self):
+        if not self.want_main_area:
+            gui.separator(self.left_side)
         self.buttonsArea = gui.widgetBox(
-            self.left_side, addSpace=0, spacing=9,
+            self.left_side, spacing=6,
             orientation=self.buttons_area_orientation,
             sizePolicy=(QSizePolicy.MinimumExpanding,
                         QSizePolicy.Maximum)
@@ -517,7 +523,7 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
 
     def _insert_main_area(self):
         self.mainArea = gui.vBox(
-            self.__splitter, margin=4,
+            self.__splitter, spacing=6,
             sizePolicy=QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
         )
         self.__splitter.addWidget(self.mainArea)
@@ -555,7 +561,6 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
         if not self.resizing_enabled:
             self.layout().setSizeConstraint(QVBoxLayout.SetFixedSize)
 
-        self.want_main_area = self.want_main_area or self.graph_name
         self._create_default_buttons()
 
         self._insert_splitter()
@@ -888,7 +893,7 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
             geom.x() + diffx, geom.y(), geom.width() + diffw, geom.height()
         )
         # bound/move by available geometry
-        bounds = QApplication.desktop().availableGeometry(self)
+        bounds = self.screen().availableGeometry()
         bounds = bounds.adjusted(
             framemargins.left(), framemargins.top(),
             -framemargins.right(), -framemargins.bottom()
@@ -937,8 +942,8 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
             restored = self.restoreGeometry(geometry)
 
         if restored and not self.windowState() & \
-                            (Qt.WindowMaximized | Qt.WindowFullScreen):
-            space = QApplication.desktop().availableGeometry(self)
+                (Qt.WindowMaximized | Qt.WindowFullScreen):
+            space = self.screen().availableGeometry()
             frame, geometry = self.frameGeometry(), self.geometry()
 
             # Fix the widget size to fit inside the available space
@@ -965,6 +970,37 @@ class OWBaseWidget(QDialog, OWComponent, Report, ProgressBarMixin,
             # Note: This should always be stored as bytes and not QByteArray.
             self.savedWidgetGeometry = bytes(self.saveGeometry())
 
+    def sizeHint(self):
+        if not self.want_basic_layout \
+                or self.mainArea_width_height_ratio is None:
+            return super().sizeHint()
+
+        # Super sizeHint with scroll_area isn't calculated right (slightly too small on macOS)
+        # on some platforms. This way, width/height should be optimal for most widgets.
+        sh = QSize()
+        # boxes look nice on macOS with starting width/height 4
+        width = 4
+        height = 4
+
+        if self.want_message_bar:
+            msh = self.statusBar().sizeHint()
+            height += msh.height()
+        if self.want_control_area:
+            csh = self.controlArea.sizeHint()
+            width += csh.width()
+            height += csh.height()
+            if self.buttons_area_orientation:
+                bsh = self.buttonsArea.sizeHint()
+                height += bsh.height()
+        height = max(height, 500)
+        if self.want_main_area:
+            width += self.__splitter.handleWidth()
+            if self.want_control_area:
+                width += int(height * self.mainArea_width_height_ratio)
+            else:
+                return super().sizeHint()
+        return QSize(width, height)
+
     # when widget is resized, save the new width and height
     def resizeEvent(self, event):
         """Overloaded to save the geometry (width and height) when the widget
